@@ -115,14 +115,6 @@ static NautilusThumbnailInfo *currently_thumbnailing = NULL;
 
 static GnomeDesktopThumbnailFactory *thumbnail_factory = NULL;
 
-static const char *types [] = {
-	"image/x-bmp", "image/x-ico", "image/jpeg", "image/gif",
-	"image/png", "image/pnm", "image/ras", "image/tga",
-	"image/tiff", "image/wbmp", "image/bmp", "image/x-xbitmap",
-	"image/x-xpixmap"
-};
-static GHashTable *image_mime_types = NULL;
-
 static int thumbnail_icon_size = 0;
 
 static gboolean
@@ -130,18 +122,24 @@ get_file_mtime (const char *file_uri, time_t* mtime)
 {
 	GFile *file;
 	GFileInfo *info;
+	gboolean ret;
 
+	ret = FALSE;
 	*mtime = INVALID_MTIME;
 
 	file = g_file_new_for_uri (file_uri);
 	info = g_file_query_info (file, G_FILE_ATTRIBUTE_TIME_MODIFIED, 0, NULL, NULL);
 	if (info) {
-		*mtime = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
+		if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_TIME_MODIFIED)) {
+			*mtime = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
+			ret = TRUE;
+		}
+
 		g_object_unref (info);
 	}
 	g_object_unref (file);
-	
-	return TRUE;
+
+	return ret;
 }
 
 static void
@@ -210,18 +208,14 @@ nautilus_update_thumbnail_file_copied (const char *source_file_uri,
 				       const char *destination_file_uri)
 {
 	char *old_thumbnail_path;
+	time_t mtime;
 	GdkPixbuf *pixbuf;
-	GFileInfo *file_info;
 	GnomeDesktopThumbnailFactory *factory;
-	GFile *destination_file;
 	
 	old_thumbnail_path = gnome_desktop_thumbnail_path_for_uri (source_file_uri, GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL);
 	if (old_thumbnail_path != NULL &&
 	    g_file_test (old_thumbnail_path, G_FILE_TEST_EXISTS)) {
-		destination_file = g_file_new_for_uri (destination_file_uri);
-		file_info = g_file_query_info (destination_file, G_FILE_ATTRIBUTE_TIME_MODIFIED, 0, NULL, NULL);
-		g_object_unref (destination_file);
-		if (file_info != NULL) {
+		if (get_file_mtime (destination_file_uri, &mtime)) {
 			pixbuf = gdk_pixbuf_new_from_file (old_thumbnail_path, NULL);
 			
 			if (pixbuf && gnome_desktop_thumbnail_has_uri (pixbuf, source_file_uri)) {
@@ -229,13 +223,12 @@ nautilus_update_thumbnail_file_copied (const char *source_file_uri,
 				gnome_desktop_thumbnail_factory_save_thumbnail (factory,
 										pixbuf,
 										destination_file_uri,
-										g_file_info_get_attribute_uint64 (file_info, G_FILE_ATTRIBUTE_TIME_MODIFIED));
+										mtime);
 			}
 			
 			if (pixbuf) {
 				g_object_unref (pixbuf);
 			}
-			g_object_unref (file_info);
 		}
 	}
 
@@ -467,7 +460,12 @@ get_pixbuf_from_data (const unsigned char *buffer,
 	if (!gdk_pixbuf_loader_close (loader, &error) ||
 	    /* Seems we have to check this even if it returned TRUE (#403255) */
 	    error != NULL) {
-		g_message ("Failed to close thumbnail pixbuf loader for %s: %s", path, error->message);
+		/* In some cases, we don't get an error even with FALSE returns (#538888) */
+		if (error != NULL) {
+			g_message ("Failed to close thumbnail pixbuf loader for %s: %s", path, error->message);
+		} else {
+			g_message ("Failed to close thumbnail pixbuf loader for %s", path);
+		}
 
 		g_object_unref (G_OBJECT (loader));
 		g_error_free (error);
@@ -740,15 +738,28 @@ thumbnail_thread_notify_file_changed (gpointer image_uri)
 static GHashTable *
 get_types_table (void)
 {
+	static GHashTable *image_mime_types = NULL;
+	GSList *format_list, *l;
+	char **types;
 	int i;
 
 	if (image_mime_types == NULL) {
 		image_mime_types = g_hash_table_new (g_str_hash, g_str_equal);
-		for (i = 0; i < G_N_ELEMENTS (types); i++) {
-			g_hash_table_insert (image_mime_types,
-					     (gpointer) types [i],
-					     GUINT_TO_POINTER (1));
+
+		format_list = gdk_pixbuf_get_formats ();
+		for (l = format_list; l; l = l->next) {
+			types = gdk_pixbuf_format_get_mime_types (l->data);
+
+			for (i = 0; i < G_N_ELEMENTS (types); i++) {
+				g_hash_table_insert (image_mime_types,
+						     types [i],
+						     GUINT_TO_POINTER (1));
+			}
+
+			g_free (types);
 		}
+
+		g_slist_free (format_list);
 	}
 
 	return image_mime_types;
