@@ -29,7 +29,6 @@
 #include "nautilus-undo-private.h"
 #include "nautilus-undo-transaction.h"
 #include <gtk/gtk.h>
-#include <libgnomecanvas/gnome-canvas.h>
 
 #define NAUTILUS_UNDO_MANAGER_DATA "Nautilus undo manager"
 
@@ -141,7 +140,7 @@ nautilus_undo_get_undo_manager (GObject *start_object)
 	GtkWindow *transient_parent;
 
 	if (start_object == NULL) {
-		return CORBA_OBJECT_NIL;
+		return NULL;
 	}
 
 	g_return_val_if_fail (G_IS_OBJECT (start_object), NULL);
@@ -174,14 +173,6 @@ nautilus_undo_get_undo_manager (GObject *start_object)
 		}
 	}
 	
-	/* In the case of a canvas item, try the canvas. */
-	if (GNOME_IS_CANVAS_ITEM (start_object)) {
-		manager = nautilus_undo_get_undo_manager (G_OBJECT (GNOME_CANVAS_ITEM (start_object)->canvas));
-		if (manager != NULL) {
-			return manager;
-		}
-	}
-	
 	/* Found nothing. I can live with that. */
 	return NULL;
 }
