@@ -587,6 +587,9 @@ add_root_for_mount (FMTreeView *view,
 	GFile *root;
 	GIcon *icon;
 
+	if (g_mount_is_shadowed (mount))
+		return;
+
 	icon = g_mount_get_icon (mount);
 	root = g_mount_get_root (mount);
 	mount_uri = g_file_get_uri (root);
@@ -735,6 +738,9 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 			/* TODO: show both unmount and eject if there are more than one volume for the drive */
 			show_unmount = g_mount_can_unmount (mount);
 			show_eject = g_mount_can_eject (mount);
+			if (show_eject) {
+				show_unmount = FALSE;
+			}
 		} 
 		
 		if (show_unmount) {
